@@ -30,8 +30,8 @@
 extern "C" {
 #endif
 
-#define BLKID_VERSION   "2.26.0"
-#define BLKID_DATE      "13-Mar-2015"
+#define BLKID_VERSION   "2.28."
+#define BLKID_DATE      "12-Apr-2016"
 
 /**
  * blkid_dev:
