@@ -25,6 +25,12 @@ static const char *lib_features[] = {
 #ifdef HAVE_SMACK
 	"smack",
 #endif
+#ifdef HAVE_BTRFS_SUPPORT
+	"btrfs",
+#endif
+#ifdef USE_LIBMOUNT_FORCE_MOUNTINFO
+	"force-mountinfo",
+#endif
 #if !defined(NDEBUG)
 	"assert",	/* libc assert.h stuff */
 #endif
@@ -99,7 +105,7 @@ int mnt_get_library_features(const char ***features)
 }
 
 #ifdef TEST_PROGRAM
-int test_version(struct libmnt_test *ts, int argc, char *argv[])
+static int test_version(struct libmnt_test *ts, int argc, char *argv[])
 {
 	const char *ver;
 	const char **features;
